/*
 *  Pedometer - Android App
 *  Copyright (C) 2009 Levente Bagi
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package name.bagi.levente.pedometer;
import sparta.checkers.quals.FlowPermission;
import sparta.checkers.quals.Sink;
import sparta.checkers.quals.FlowPermission;
import sparta.checkers.quals.Source;


/**
 * Calculates and displays the distance walked.  
 * @author Levente Bagi
 */
public class DistanceNotifier implements StepListener, SpeakingTimer.Listener {

    public interface Listener {
        public void valueChanged(@Source({FlowPermission.SHARED_PREFERENCES}) @Sink({FlowPermission.SHARED_PREFERENCES}) float value);
        public void passValue();
    }
    private @Source({}) @Sink({}) Listener mListener;
    
    @Source({FlowPermission.SHARED_PREFERENCES})
    @Sink({FlowPermission.SPEAKER,FlowPermission.SHARED_PREFERENCES})
    float mDistance = 0;
    
    @Source({})
    @Sink({})
    PedometerSettings mSettings;
    @Source({})
    @Sink({})
    Utils mUtils;
    
    @Source({FlowPermission.SHARED_PREFERENCES})
    @Sink({})
    boolean mIsMetric;
    @Source({FlowPermission.SHARED_PREFERENCES})
    @Sink({FlowPermission.SPEAKER,FlowPermission.SHARED_PREFERENCES})
    float mStepLength;

    public DistanceNotifier(@Source({}) @Sink({}) Listener listener, @Source({}) @Sink({}) PedometerSettings settings, @Source({}) @Sink({}) Utils utils) {
        mListener = listener;
        mUtils = utils;
        mSettings = settings;
        reloadSettings();
    }
    public void setDistance(@Source({FlowPermission.SHARED_PREFERENCES}) @Sink({FlowPermission.SPEAKER,FlowPermission.SHARED_PREFERENCES}) float distance) {
        mDistance = distance;
        notifyListener();
    }
    
    public void reloadSettings() {
        mIsMetric = mSettings.isMetric();
        mStepLength = mSettings.getStepLength();
        notifyListener();
    }
    
    public void onStep() {
        
        if (mIsMetric) {
            mDistance += (float)(// kilometers
                mStepLength // centimeters
                / 100000.0); // centimeters/kilometer
        }
        else {
            mDistance += (float)(// miles
                mStepLength // inches
                / 63360.0); // inches/mile
        }
        
        notifyListener();
    }
    
    private void notifyListener() {
        mListener.valueChanged(mDistance);
    }
    
    public void passValue() {
        // Callback of StepListener - Not implemented
    }

    public void speak() {
        if (mSettings.shouldTellDistance()) {
            if (mDistance >= .001f) {
                mUtils.say(("" + (mDistance + 0.000001f)).substring(0, 4) + (mIsMetric ? " kilometers" : " miles"));
                // TODO: format numbers (no "." at the end)
            }
        }
    }
    

}

