package name.bagi.levente.pedometer.preferences;

import sparta.checkers.quals.FlowPermission;
import sparta.checkers.quals.Sink;
import sparta.checkers.quals.FlowPermission;
import sparta.checkers.quals.Source;
import android.content.Context;
import android.os.Bundle;
import android.preference.EditTextPreference;
import android.preference.PreferenceManager;
import android.text.InputType;
import android.util.AttributeSet;
import android.view.View;
import android.widget.EditText;

/**
 * An {@link EditTextPreference} that is suitable for entering measurements.
 * It handles metric/imperial setting.
 * @author Levente Bagi
 */
abstract public class EditMeasurementPreference extends EditTextPreference {
	@Source({FlowPermission.SHARED_PREFERENCES})
	@Sink({})
	boolean mIsMetric;
	
	protected @Source({}) @Sink({}) int mTitleResource;
	protected @Source({}) @Sink({}) int mMetricUnitsResource;
	protected @Source({}) @Sink({}) int mImperialUnitsResource;
	
	public EditMeasurementPreference(@Source({}) @Sink({}) Context context) {
		super(context);
		initPreferenceDetails();
	}
	public EditMeasurementPreference(@Source({}) @Sink({}) Context context, @Source({}) @Sink({}) AttributeSet attr) {
		super(context, attr);
		initPreferenceDetails();
	}
	public EditMeasurementPreference(@Source({}) @Sink({}) Context context, @Source({}) @Sink({}) AttributeSet attr, @Source({}) @Sink({}) int defStyle) {
		super(context, attr, defStyle);
		initPreferenceDetails();
	}
	
	abstract protected void initPreferenceDetails();
	
	protected void showDialog(@Source({FlowPermission.ANY}) @Sink({}) Bundle state) {
		mIsMetric = PreferenceManager.getDefaultSharedPreferences(getContext()).getString("units", "imperial").equals("metric");
		setDialogTitle(
				getContext().getString(mTitleResource) + 
				" (" + 
						getContext().getString(
								mIsMetric
								? mMetricUnitsResource 
								: mImperialUnitsResource) + 
				")"
		);
		
		try {
			Float.valueOf(getText());
		}
		catch (Exception e) {
			setText("20");
		}
		
		super.showDialog(state);
	}
	protected void onAddEditTextToDialogView (@Source({FlowPermission.ANY}) @Sink({}) View dialogView, @Source({FlowPermission.ANY}) @Sink({}) EditText editText) {
		editText.setRawInputType(
				InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
		super.onAddEditTextToDialogView(dialogView, editText);
	}
	public void onDialogClosed(@Source({FlowPermission.ANY}) @Sink({}) boolean positiveResult) {
		if (positiveResult) {
			try {
				Float.valueOf(((CharSequence)(getEditText().getText())).toString());
			}
			catch (NumberFormatException e) {
				this.showDialog(null);
				return;
			}
		}
		super.onDialogClosed(positiveResult);
	}
}
